@@ -47,7 +47,9 @@ def ptn_rnn_decoder(cell,
                     mode="SAMPLE",
                     reuse=False,
                     beam_size=None,
-                    temperature=1):
+                    temperature=1,
+                    train_sample='random',
+                    predict_sample='random'):
     """
     :param cell:
     :param decoder_target_ids:
@@ -192,9 +194,9 @@ def ptn_rnn_decoder(cell,
             return tf.gather_nd(enc_outputs, input_index_pairs)
 
         def custom_sample_from_logits(logits):
-            _, top4 = tf.nn.top_k(logits, 4)
-            prob = tf.constant([[4, 3, 2, 1]],dtype=tf.float32)
-            sample = tf.cast(tf.multinomial(logits=tf.tile(prob, [batch_size, 1]), num_samples=1),dtype=tf.int32)
+            _, top4 = tf.nn.top_k(logits, 5)
+            prob = tf.constant([[5, 4, 3, 2, 1]], dtype=tf.float32)
+            sample = tf.cast(tf.multinomial(logits=tf.tile(prob, [batch_size, 1]), num_samples=1), dtype=tf.int32)
             sampled_idx = tf.gather_nd(top4, tf.concat([tf.range(batch_size)[:, tf.newaxis], sample], axis=-1))
 
             return sampled_idx
@@ -251,7 +253,7 @@ def ptn_rnn_decoder(cell,
         # region 更新point_mask
         selected_mask = tf.cast(tf.ones([batch_size, seq_length], dtype=tf.int32), tf.bool)
         l_mask = l1_mask
-        t_mask = tf.math.logical_or(t3_mask, t4_mask)
+        t_mask = tf.math.logical_or(t2_mask, t4_mask)
         point_mask = tf.math.logical_and(selected_mask, tf.math.logical_and(l_mask, t_mask))
         # endregion
 
@@ -260,8 +262,8 @@ def ptn_rnn_decoder(cell,
         output_logits = [logits]
         if (mode in ['SAMPLE', "GREEDY", "CUSTOM"]):
             if mode == "SAMPLE":
-                # sample_fn = random_sample_from_logits
-                sample_fn = custom_sample_from_logits
+                sample_fn = random_sample_from_logits
+                # sample_fn = custom_sample_from_logits
 
                 if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
                                        tf.ones([batch_size, seq_length], dtype=tf.bool),
@@ -269,25 +271,38 @@ def ptn_rnn_decoder(cell,
                 if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
                                           tf.ones([batch_size, seq_length], dtype=tf.bool),
                                           tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+                if_l2_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                             tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                             tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
             elif mode == "GREEDY":
                 sample_fn = greedy_sample_from_logits
-                if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.1),
-                                       tf.ones([batch_size, seq_length], dtype=tf.bool),
-                                       tf.tile(tf.math.logical_not(l0_ssr_mask)[tf.newaxis, :], [batch_size, 1]))
-                if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
-                                          tf.ones([batch_size, seq_length], dtype=tf.bool),
-                                          tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
-            elif mode == 'CUSTOM':
-                # sample_fn = custom_sample_from_logits
-                sample_fn = random_sample_from_logits
+
                 if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.1),
                                        tf.ones([batch_size, seq_length], dtype=tf.bool),
                                        tf.tile(tf.math.logical_not(l0_ssr_mask)[tf.newaxis, :], [batch_size, 1]))
                 if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.9),
                                           tf.ones([batch_size, seq_length], dtype=tf.bool),
                                           tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+                if_l2_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                             tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                             tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+            elif mode == 'CUSTOM':
+                sample_fn = random_sample_from_logits
+                # sample_fn = custom_sample_from_logits
+
+                if_ssr_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.1),
+                                       tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                       tf.tile(tf.math.logical_not(l0_ssr_mask)[tf.newaxis, :], [batch_size, 1]))
+                if_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 1.0),
+                                          tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                          tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+                if_l2_taohua_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                             tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                             tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis, :], [batch_size, 1]))
+
+                taohua_ajust_mask = tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.0)
             else:
-                raise NotImplementedError("invalid mode: %s. Available modes: [SAMPLE, GREEDY]" % mode)
+                raise NotImplementedError("invalid mode: %s. Available modes: [SAMPLE, GREEDY, CUSTOM]" % mode)
 
             output_idx = sample_fn(logits)  # [batch_size]
             output_idxs = [output_idx]
@@ -299,28 +314,40 @@ def ptn_rnn_decoder(cell,
 
                 if i == 1:
                     l_mask = l1_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                      tf.tile(t2_mask[tf.newaxis, :], [batch_size, 1]),
+                                      tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
                 elif i == 2:
                     l_mask = l1_mask
-                    t_mask = t0_mask
+                    t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                      tf.tile(t3_mask[tf.newaxis, :], [batch_size, 1]),
+                                      tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
                 elif i == 3:
                     l_mask = l2_mask
                     t_mask = t1_mask
                 elif i == 4:
                     l_mask = l2_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                      tf.tile(t2_mask[tf.newaxis, :], [batch_size, 1]),
+                                      tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
                 elif i == 5:
                     l_mask = l2_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                      tf.tile(t3_mask[tf.newaxis, :], [batch_size, 1]),
+                                      tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
                 elif i == 6:
                     l_mask = l3_mask
                     t_mask = t1_mask
                 elif i == 7:
                     l_mask = l3_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                      tf.tile(t2_mask[tf.newaxis, :], [batch_size, 1]),
+                                      tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
                 elif i == 8:
                     l_mask = l3_mask
-                    t_mask = tf.math.logical_or(t3_mask, t4_mask)
+                    t_mask = tf.where(tf.less(tf.random_uniform([batch_size], 0, 1, dtype=tf.float32), 0.5),
+                                      tf.tile(t3_mask[tf.newaxis, :], [batch_size, 1]),
+                                      tf.tile(t4_mask[tf.newaxis, :], [batch_size, 1]))
                 else:
                     raise Exception('update point_mask failed, unexcepted i')
 
@@ -328,15 +355,17 @@ def ptn_rnn_decoder(cell,
                 # endregion
                 ssr_mask = tf.math.logical_not(tf.math.logical_and(tf.reduce_any(tf.math.logical_and(l0_ssr_mask, tf.math.logical_not(selected_mask)), axis=1, keep_dims=True), l0_ssr_mask))
                 th_mask = tf.math.logical_not(tf.math.logical_and(tf.reduce_any(tf.math.logical_and(taohua_mask, tf.math.logical_not(selected_mask)), axis=1, keep_dims=True), taohua_mask))
-
-                # xx = tf.reduce_any(tf.math.logical_and(l0_ssr_mask, tf.math.logical_not(selected_mask)), axis=1)
-                # ssr_mask=tf.where(xx, tf.tile(tf.math.logical_not(l0_ssr_mask)[tf.newaxis],[batch_size,1]), ones)
-                # xx = tf.reduce_any(tf.math.logical_and(taohua_mask, tf.math.logical_not(selected_mask)), axis=1)
-                # th_mask = tf.where(xx, tf.tile(tf.math.logical_not(taohua_mask)[tf.newaxis],[batch_size,1]), ones)
+                if mode == 'CUSTOM':
+                    th_mask = tf.where(taohua_ajust_mask,
+                                       tf.ones([batch_size, seq_length], dtype=tf.bool),
+                                       th_mask)
 
                 point_mask = tf.math.logical_and(point_mask, tf.math.logical_and(th_mask, ssr_mask))
                 point_mask = tf.math.logical_and(point_mask, tf.math.logical_and(if_ssr_mask, if_taohua_mask))
 
+                if i in [3, 4, 5]:
+                    point_mask = tf.math.logical_and(point_mask, if_l2_taohua_mask)
+
                 logits, state = call_cell(output_idx, state, point_mask)  # [batch_size, data_len]
                 output_logits.append(logits)
                 output_idx = sample_fn(logits)  # [batch_size]
@@ -354,25 +383,25 @@ def ptn_rnn_decoder(cell,
 
                 if i == 1:
                     l_mask = l1_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.math.logical_or(t2_mask, t4_mask)
                 elif i == 2:
                     l_mask = l1_mask
-                    t_mask = t0_mask
+                    t_mask = tf.math.logical_or(t3_mask, t4_mask)
                 elif i == 3:
                     l_mask = l2_mask
                     t_mask = t1_mask
                 elif i == 4:
                     l_mask = l2_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.math.logical_or(t2_mask, t4_mask)
                 elif i == 5:
                     l_mask = l2_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.math.logical_or(t3_mask, t4_mask)
                 elif i == 6:
                     l_mask = l3_mask
                     t_mask = t1_mask
                 elif i == 7:
                     l_mask = l3_mask
-                    t_mask = tf.math.logical_or(t2_mask, tf.math.logical_or(t3_mask, t4_mask))
+                    t_mask = tf.math.logical_or(t2_mask, t4_mask)
                 elif i == 8:
                     l_mask = l3_mask
                     t_mask = tf.math.logical_or(t3_mask, t4_mask)
