@@ -116,55 +116,56 @@ class FeatureUtil():
     def to_tfrecord(self, csvfile, filename):
         writer = tf.python_io.TFRecordWriter(filename)
         data = open(csvfile, 'r', encoding='utf8').read().split('\n')[1:-1]
-        role_id_hashs, sequence_ids, sequence_times, sequence_time_gapss, cross_features_indexs, cross_features_vals, user_features_ids, \
-        pos_labels, neg_labels, masks, cur_times = self.feature_extraction(data)
         print('feature_extraction')
-        for i in range(len(data)):
-            role_id_hash = role_id_hashs[i]
-            sequence_id = sequence_ids[i]
-            sequence_time = sequence_times[i]
-            sequence_time_gaps = sequence_time_gapss[i]
-            cross_features_index = cross_features_indexs[i]
-            cross_features_val = cross_features_vals[i]
-            user_features_id = user_features_ids[i]
-            pos_label = pos_labels[i]
-            neg_label = neg_labels[i]
-            mask = masks[i]
-            cur_time = cur_times[i]
+        for jj in range(1+len(data)//10000):
+            role_id_hashs, sequence_ids, sequence_times, sequence_time_gapss, cross_features_indexs, cross_features_vals, user_features_ids, \
+            pos_labels, neg_labels, masks, cur_times = self.feature_extraction(data[jj*10000 : (jj+1)*10000])
+            for i in range(len(role_id_hashs)):
+                role_id_hash = role_id_hashs[i]
+                sequence_id = sequence_ids[i]
+                sequence_time = sequence_times[i]
+                sequence_time_gaps = sequence_time_gapss[i]
+                cross_features_index = cross_features_indexs[i]
+                cross_features_val = cross_features_vals[i]
+                user_features_id = user_features_ids[i]
+                pos_label = pos_labels[i]
+                neg_label = neg_labels[i]
+                mask = masks[i]
+                cur_time = cur_times[i]
 
-            sequence_id_feature = [tf.train.Feature(int64_list=tf.train.Int64List(value=sequence_id[i])) for i in range(self.seq_num)]
-            sequence_time_feature = [tf.train.Feature(int64_list=tf.train.Int64List(value=sequence_time[i])) for i in range(self.seq_num)]
-            sequence_time_gaps_feature = [tf.train.Feature(int64_list=tf.train.Int64List(value=sequence_time_gaps[i])) for i in range(self.seq_num)]
+                sequence_id_feature = [tf.train.Feature(int64_list=tf.train.Int64List(value=sequence_id[i])) for i in range(self.seq_num)]
+                sequence_time_feature = [tf.train.Feature(int64_list=tf.train.Int64List(value=sequence_time[i])) for i in range(self.seq_num)]
+                sequence_time_gaps_feature = [tf.train.Feature(int64_list=tf.train.Int64List(value=sequence_time_gaps[i])) for i in range(self.seq_num)]
 
-            role_id_hash_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=[role_id_hash]))
-            cross_features_index_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=cross_features_index))
-            cross_features_val_feature = tf.train.Feature(float_list=tf.train.FloatList(value=cross_features_val))
-            user_features_id_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=user_features_id))
-            pos_label_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=pos_label))
-            neg_label_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=neg_label))
-            mask_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=mask))
-            cur_time_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=[cur_time]))
+                role_id_hash_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=[role_id_hash]))
+                cross_features_index_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=cross_features_index))
+                cross_features_val_feature = tf.train.Feature(float_list=tf.train.FloatList(value=cross_features_val))
+                user_features_id_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=user_features_id))
+                pos_label_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=pos_label))
+                neg_label_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=neg_label))
+                mask_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=mask))
+                cur_time_feature = tf.train.Feature(int64_list=tf.train.Int64List(value=[cur_time]))
 
-            feature = {
-                # 'hist_num': hist_num_feature,
-                'role_id_hash': role_id_hash_feature,
-                "cross_features_index": cross_features_index_feature,
-                "cross_features_val": cross_features_val_feature,
-                "user_features_id": user_features_id_feature,
-                "pos_label": pos_label_feature,
-                "neg_label": neg_label_feature,
-                'mask': mask_feature,
-                'cur_time': cur_time_feature,
-            }
-            for seq_num_i in range(self.seq_num):
-                feature['sequence_id_' + str(seq_num_i)] = sequence_id_feature[seq_num_i]
-                feature['sequence_time_' + str(seq_num_i)] = sequence_time_feature[seq_num_i]
-                feature['sequence_time_gaps_' + str(seq_num_i)] = sequence_time_gaps_feature[seq_num_i]
+                feature = {
+                    # 'hist_num': hist_num_feature,
+                    'role_id_hash': role_id_hash_feature,
+                    "cross_features_index": cross_features_index_feature,
+                    "cross_features_val": cross_features_val_feature,
+                    "user_features_id": user_features_id_feature,
+                    "pos_label": pos_label_feature,
+                    "neg_label": neg_label_feature,
+                    'mask': mask_feature,
+                    'cur_time': cur_time_feature,
+                }
+                for seq_num_i in range(self.seq_num):
+                    feature['sequence_id_' + str(seq_num_i)] = sequence_id_feature[seq_num_i]
+                    feature['sequence_time_' + str(seq_num_i)] = sequence_time_feature[seq_num_i]
+                    feature['sequence_time_gaps_' + str(seq_num_i)] = sequence_time_gaps_feature[seq_num_i]
 
-            seq_example = tf.train.Example(
-                features=tf.train.Features(feature=feature)
-            )
-            writer.write(seq_example.SerializeToString())
+                seq_example = tf.train.Example(
+                    features=tf.train.Features(feature=feature)
+                )
+                writer.write(seq_example.SerializeToString())
         writer.close()
 
     def read_tfrecord(self, filename, is_pred=False):
